REQUIRE_JS = ('https://cdnjs.cloudflare.com/ajax/libs/require.js/2.1.20/'
              'require.js')
BOOTSTRAP = 'https://bootswatch.com/flatly/bootstrap.min.css'

twitter_api_key = 'twitter API key of your registered twitter application'
twitter_api_secret = 'consumer secret'
twitter_access_token = 'Access token'
twitter_access_secret = 'Access token secret'

twitter_stream_filter = {'track': 'python'}
